"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.CustomLogger = CustomLogger;
exports.LogLevel = exports.default = void 0;
var logger = console;
var LogLevel = {
  silent: 0,
  error: 1,
  warn: 2,
  info: 3,
  debug: 4
};
exports.LogLevel = LogLevel;

function loggerFunctionMaker(logLevel) {
  return function (type) {
    return function () {
      if (logLevel >= LogLevel[type]) {
        logger[type].apply(logger, arguments);
      }
    };
  };
}

function CustomLogger(logLevel) {
  var loggerTypes = ['error', 'warn', 'info', 'debug'];
  return loggerTypes.reduce(function (funcObj, type) {
    funcObj[type] = loggerFunctionMaker(logLevel)(type);
    return funcObj;
  }, {});
}

var level = LogLevel[process.env.LOG_LEVEL];

if (level == null) {
  level = process.env.NODE_ENV === 'production' ? LogLevel.error : LogLevel.debug;
}

var log = new CustomLogger(level);
exports.default = log;