"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.basename = basename;
exports.foldername = foldername;
exports.getRegExpFromRequireContext = getRegExpFromRequireContext;
exports.isFileNameCorrect = isFileNameCorrect;
exports.isFilePathCorrect = isFilePathCorrect;
exports.applySubFuncs = applySubFuncs;
exports.isRequireContextRegExpPassed = isRequireContextRegExpPassed;
exports.flattenContentObj = flattenContentObj;
exports.getComponent = getComponent;
exports.applyFuncList = applyFuncList;
exports.unaryFunc = unaryFunc;
exports.toList = toList;
exports.toReact = toReact;

var _curry2 = _interopRequireDefault(require("ramda/src/curry"));

var _2 = _interopRequireDefault(require("ramda/src/__"));

var _pipe2 = _interopRequireDefault(require("ramda/src/pipe"));

var _toPairs2 = _interopRequireDefault(require("ramda/src/toPairs"));

var _react = _interopRequireDefault(require("react"));

var _path = _interopRequireDefault(require("path"));

var _logger = _interopRequireDefault(require("./logger"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

// const allExtRegExp = /\.[0-9a-z]+$/i;
var allExtRegExp = /(.*?)/;
/**
 * Return the basename of the file
 *
 * @param p the path to evaluate
 * @param regExp optionally, regex for remove
 */

function basename(filePath) {
  var regExp = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

  var fileName = _path.default.basename(filePath);

  if (regExp) {
    fileName = fileName.replace(regExp, '');
  }

  return fileName;
}
/**
 * Return the folder name.
 * If folder is relative, dot will be retrieved.
 * @example ./path.1/path2/filename  =>  path.1/path2
 * @example /path.1/path2/filename  =>  path.1/path2
 *
 * @param p the path to evaluate
 */


function foldername(filePath) {
  return _path.default.dirname(filePath).replace(/^.?\//, '');
}

function getRegExpStrFromRequireContext(req) {
  var id = req.id;

  if (id == null) {
    return null;
  }

  var regExpStr = id.split(' ').pop();

  if (regExpStr === '') {
    return null;
  }

  return regExpStr;
}
/**
 * Return RegExp string in require.context instance
 *
 * @param {*require.context} req webpack's require.context instance
 */


function getRegExpFromRequireContext(req) {
  var regExpStr = getRegExpStrFromRequireContext(req);
  return regExpStr != null ? new RegExp(regExpStr) : null;
}
/**
 * Return whether regexp is passed to require.context or not.
 *
 * @param {*require.context} req webpack's require.context instance
 */


function isRequireContextRegExpPassed(req) {
  return getRegExpStrFromRequireContext(req) !== '^\\.\\/.*$';
}
/**
 * Return true if file's name match RegExp
 *
 * @param {*string} filePath
 * @param {*RegExp} regExp
 */


function isFileNameCorrect(filePath, includeRegExp) {
  var excludeRegExp = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

  var fileName = _path.default.basename(filePath);

  return (includeRegExp || allExtRegExp).test(fileName) && (excludeRegExp == null ? true : !excludeRegExp.test(fileName));
}
/**
 * Return true if file's full path(include filename) match RegExp
 *
 * @param {*string} filePath
 * @param {*RegExp} regExp
 */


function isFilePathCorrect(filePath, includeRegExp) {
  var excludeRegExp = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;
  return (includeRegExp || allExtRegExp).test(filePath) && (excludeRegExp == null ? true : !excludeRegExp.test(filePath));
}
/**
 * Apply all functions in funcObj to baseObj.
 * @example funcList: [ "func1", ["func2"], ["func3", [ param1, param2 ]];
 * will call -- baseObj.func1() -> baseObj.func2() -> baseObj.func3(param1, param2);
 *
 * @param {*instance} baseObj main function
 * @param {*object} funcObj sub function object
 */


function applySubFuncs(baseObj) {
  var funcList = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  funcList.forEach(function (funcInfo) {
    if (typeof funcInfo === 'string') {
      // only sub function
      baseObj[funcInfo] && baseObj[funcInfo]();
    } else {
      var _funcInfo = _slicedToArray(funcInfo, 2),
          funcName = _funcInfo[0],
          params = _funcInfo[1];

      if (params == null) {
        // run sub function without params
        baseObj[funcName] && baseObj[funcName]();
        return;
      }

      if (!Array.isArray(params)) {
        _logger.default.error("function [".concat(funcName, "]'s parameters in storySubFuncList MUST be an array."));

        return;
      }

      baseObj[funcName] && baseObj[funcName].apply(baseObj, _toConsumableArray(params));
    }
  });
}
/**
 * Return flattened array of contentObj
 * @example { a1: '111', b1: '222' }  ->  [['', [['a1', '111'], ['b1', '222']]]]
 * @example { fa: { a1: '111', a2: '112'}, fb: { b1: '222'}}  ->  [["fa", [["a1", "111"], ["a2", "112"]]], ["fb", [["b1", "222"]]]]
 *
 * @param contentObj object of filecontents loaded by loader.js
 * @param containGroupFolder boolean
 */


function flattenContentObj(contentObj) {
  var containGroupFolder = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
  // flatten folder layer
  var folderBaseContentList;

  if (containGroupFolder) {
    folderBaseContentList = (0, _toPairs2.default)(contentObj).map(function (_ref) {
      var _ref2 = _slicedToArray(_ref, 2),
          folderName = _ref2[0],
          fileContentObj = _ref2[1];

      return [folderName, (0, _toPairs2.default)(fileContentObj)];
    });
  } else {
    // if folder layer is not exist, add a space folder
    folderBaseContentList = [['', (0, _toPairs2.default)(contentObj)]];
  }

  return folderBaseContentList;
}
/**
 * Return react Component(function) from fileContent
 *
 * @param fileContent instance of req(fileName)
 */


function getComponent(fileContent) {
  if (_typeof(fileContent) === 'object' && typeof fileContent.default === 'function') {
    return fileContent.default;
  }

  return fileContent;
} // /**
//  * Return composed purefunction and skip setting the first parameter
//  *
//  * @param funcList
//  */
// function composePureFuncSkipFirstParam(funcList) {
//   const allFuncs = funcList.map(({ func, params }) => {
//     if (params == null) {
//       return func;
//     }
//     return curry(func)(__, ...params);
//   });
//   return pipe(...allFuncs);
// }

/**
 * Return component applied all functions in funcList, funcList MUST be pure
 * @example funcList: [hoc1, hoc2]
 *  will return hoc2(hoc1(component))
 *
 * @param component target Component
 * @param funcList function list
 */


function applyFuncList(component) {
  var funcList = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

  if (funcList.length === 0) {
    return component;
  }

  var composedFuncs = _pipe2.default.apply(void 0, _toConsumableArray(funcList));

  return composedFuncs(component);
}
/**
 * Return new function with setting all params from second place.
 * @example (function myName(name, msg1, msg2) {}, params=['hello', 'world'])  =>
 *   myName(name, 'hello', 'world');
 *
 * @param func
 * @param params parameter array
 */


function unaryFunc(func) {
  var params = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

  if (params == null || params.length === 0) {
    return func;
  }

  return (0, _curry2.default)(func).apply(void 0, [_2.default].concat(_toConsumableArray(params)));
}
/**
 * Change input element to list if it's not
 *
 * @param {any} elem
 */


function toList(elem) {
  if (Array.isArray(elem)) return elem;
  return [elem];
}

function toReact(Component) {
  var toReactFunc = function toReactFunc(props) {
    return _react.default.createElement(Component, props);
  };

  return toReactFunc;
}