"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _identity2 = _interopRequireDefault(require("ramda/src/identity"));

var _sort2 = _interopRequireDefault(require("ramda/src/sort"));

var _reduce2 = _interopRequireDefault(require("ramda/src/reduce"));

var _compose2 = _interopRequireDefault(require("ramda/src/compose"));

var _util = require("./lib/util");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var defaultOptions = {
  sort: true,
  sortFunc: function sortFunc(a, b) {
    return a.localeCompare(b);
  },
  noExt: true,
  noExtRegExp: null,
  includeRegExp: null,
  excludeRegExp: null,
  groupByFolder: false,
  ignoreDotFolder: true
};
/**
 * Return the list of file name and file content(by require) from a directory.
 * Content can be a string if you load *.json, *.md, or can be an object or function if you load *.js, *.jsx
 * @example return { path(without extention):  [NameOfMyComponent, SrcOfMyComponent] }
 * @example { folder1: { path1: [NameOfComponent1, SrcOfComponent1], path2: [NameOfComponent2, SrcOfComponent2] },
 * folder2: { path3: [FileNameComponent3, SrcOfComponent3]} }
 *
 * @param {*require.context} req webpack's require.context
 * @param {*object} userOptions optionally,
 */

function loader(req) {
  var userOptions = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var _defaultOptions$userO = _objectSpread({}, defaultOptions, userOptions),
      sort = _defaultOptions$userO.sort,
      sortFunc = _defaultOptions$userO.sortFunc,
      noExt = _defaultOptions$userO.noExt,
      noExtRegExp = _defaultOptions$userO.noExtRegExp,
      includeRegExp = _defaultOptions$userO.includeRegExp,
      fileExcludeRegExp = _defaultOptions$userO.excludeRegExp,
      groupByFolder = _defaultOptions$userO.groupByFolder,
      ignoreDotFolder = _defaultOptions$userO.ignoreDotFolder; // try get regexp from req


  var regExpFromReq;

  if ((0, _util.isRequireContextRegExpPassed)(req)) {
    // ignore default regexp
    regExpFromReq = (0, _util.getRegExpFromRequireContext)(req);
  }

  var fileIncludeRegExp = includeRegExp || regExpFromReq; // use requirecontext to read contents of the file

  function fileContentReducer(baseObj, filePath) {
    if (!(0, _util.isFilePathCorrect)(filePath, fileIncludeRegExp, fileExcludeRegExp)) {
      // skip if extention doesn't match regexp
      return baseObj;
    }

    var fileName = (0, _util.basename)(filePath, noExt ? noExtRegExp || fileIncludeRegExp : null);
    var folderName = (0, _util.foldername)(filePath);
    var key = "".concat(folderName, "/").concat(fileName); // add folder layer

    if (ignoreDotFolder && folderName === '.') {
      return baseObj;
    }

    if (!groupByFolder) {
      baseObj[key] = [fileName, req(filePath)];
    } else {
      baseObj[folderName] = _objectSpread({}, baseObj[folderName], _defineProperty({}, key, [fileName, req(filePath)]));
    }

    return baseObj;
  }

  return (0, _compose2.default)((0, _reduce2.default)(fileContentReducer, {}), (0, _sort2.default)(sort ? sortFunc : _identity2.default))(req.keys());
}

var _default = loader;
exports.default = _default;
module.exports = exports.default;